"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = default_1;
const debug_1 = __importDefault(require("debug"));
const promises_1 = __importDefault(require("fs/promises"));
const parseurl_1 = __importDefault(require("parseurl"));
const path_1 = __importDefault(require("path"));
const send_1 = __importStar(require("send"));
const serve_static_1 = __importDefault(require("serve-static"));
const debug = (0, debug_1.default)("connect:static-compression");
function setHeader(res, path, encoding) {
    const type = send_1.mime.lookup(path);
    const charset = send_1.mime.charsets.lookup(type, "");
    debug("content-type %s", type);
    res.setHeader("Content-Type", type + (charset ? "; charset=" + charset : ""));
    res.setHeader("Content-Encoding", encoding);
    res.setHeader("Vary", "Accept-Encoding");
}
/**
 * Creates cache of filenames matching the regex
 * @param {String} root
 * @param {RegExp} regex
 * @returns newly create cache
 */
function createCache(root, regex) {
    return __awaiter(this, void 0, void 0, function* () {
        const files = yield promises_1.default.readdir(root, { recursive: true });
        const entries = files
            .filter(n => regex.test(n))
            .map(n => path_1.default.resolve(root, n));
        debug("Found %d compressed files", entries.length);
        return new Set(entries);
    });
}
function createMethods(root) {
    return __awaiter(this, void 0, void 0, function* () {
        return [
            {
                extension: ".br",
                encoding: "br",
                cache: yield createCache(root, /\.br$/),
            },
            {
                extension: ".gz",
                encoding: "gzip",
                cache: yield createCache(root, /\.gz$/),
            },
        ];
    });
}
function default_1(root, options = {}) {
    const methodsPromise = createMethods(root);
    options.index || (options.index = "index.html");
    const setHeaders = options.setHeaders;
    const serveStatic = (0, serve_static_1.default)(root, options);
    return function gzipStatic(req, res, next) {
        return __awaiter(this, void 0, void 0, function* () {
            if ("GET" != req.method && "HEAD" != req.method) {
                return next();
            }
            let name;
            checking: {
                for (const method of yield methodsPromise) {
                    name = check(req, method);
                    if (name) {
                        break checking;
                    }
                }
                debug("Passing %s", req.url);
                return serveStatic(req, res, next);
            }
            debug("Sending %s", name.full);
            setHeader(res, name.orig, name.encoding);
            const stream = (0, send_1.default)(req, name.compressed, {
                maxAge: options.maxAge || 0,
                root,
                index: name.index,
                cacheControl: options.cacheControl,
                lastModified: options.lastModified,
                etag: options.etag,
                dotfiles: options.dotfiles,
            }).on("error", next);
            if (setHeaders) {
                stream.on("headers", setHeaders);
            }
            stream.pipe(res);
        });
    };
    function check(req, { encoding, extension, cache, }) {
        var _a, _b;
        const acceptEncoding = req.headers["accept-encoding"] || "";
        if (!acceptEncoding.includes(encoding)) {
            return;
        }
        const name = {
            orig: (_b = (_a = (0, parseurl_1.default)(req)) === null || _a === void 0 ? void 0 : _a.pathname) !== null && _b !== void 0 ? _b : "",
            encoding,
        };
        if (name.orig.at(-1) === "/") {
            name.compressed = name.orig;
            name.orig += options.index;
            name.index = options.index + extension;
        }
        else {
            name.compressed = name.orig + extension;
        }
        name.full = path_1.default.join(root, name.orig + extension);
        debug("request %s, check for %s", req.url, name.full);
        if (cache.has(name.full)) {
            return name;
        }
    }
}
//# sourceMappingURL=gzip-static.js.map